@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Test::Fatal;
 {
-  $Test::Fatal::VERSION = '0.008';
+  $Test::Fatal::VERSION = '0.012';
 }
 # ABSTRACT: incredibly simple helpers for testing code with exceptions
 
@@ -16,10 +16,32 @@ our @EXPORT    = qw(exception);
 our @EXPORT_OK = qw(exception success dies_ok lives_ok);
 
 
+our ($REAL_TBL, $REAL_CALCULATED_TBL) = (1, 1);
+
 sub exception (&) {
   my $code = shift;
 
   return try {
+    my $incremented = $Test::Builder::Level - $REAL_CALCULATED_TBL;
+    local $Test::Builder::Level = $REAL_CALCULATED_TBL;
+    if ($incremented) {
+        # each call to exception adds 5 stack frames
+        $Test::Builder::Level += 5;
+        for my $i (1..$incremented) {
+            # -2 because we want to see it from the perspective of the call to
+            # is() within the call to $code->()
+            my $caller = caller($Test::Builder::Level - 2);
+            if ($caller eq __PACKAGE__) {
+                # each call to exception adds 5 stack frames
+                $Test::Builder::Level = $Test::Builder::Level + 5;
+            }
+            else {
+                $Test::Builder::Level = $Test::Builder::Level + 1;
+            }
+        }
+    }
+
+    local $REAL_CALCULATED_TBL = $Test::Builder::Level;
     $code->();
     return undef;
   } catch {
@@ -70,6 +92,7 @@ sub lives_ok (&;$) {
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
@@ -78,7 +101,7 @@ Test::Fatal - incredibly simple helpers for testing code with exceptions
 
 =head1 VERSION
 
-version 0.008
+version 0.012
 
 =head1 SYNOPSIS
 
@@ -93,10 +116,10 @@ version 0.008
     "the code lived",
   );
 
-  isnt(
+  like(
     exception { might_die; },
-    undef,
-    "the code died",
+    qr/turns out it died/,
+    "the code died as expected",
   );
 
   isa_ok(
@@ -122,7 +145,7 @@ It exports one routine by default: C<exception>.
 C<exception> takes a bare block of code and returns the exception thrown by
 that block.  If no exception was thrown, it returns undef.
 
-B<ACHTUNG!>  If the block results in a I<false> exception, such as 0 or the
+B<Achtung!>  If the block results in a I<false> exception, such as 0 or the
 empty string, Test::Fatal itself will die.  Since either of these cases
 indicates a serious problem with the system under testing, this behavior is
 considered a I<feature>.  If you must test for these conditions, you should use
@@ -142,14 +165,30 @@ C<Sub::Uplevel> mechanism.
 
 B<Achtung!>  This is not a great idea:
 
-  like( exception { ... }, qr/foo/, "foo appears in the exception" );
+  sub exception_like(&$;$) {
+      my ($code, $pattern, $name) = @_;
+      like( &exception($code), $pattern, $name );
+  }
+
+  exception_like(sub { }, qr/foo/, 'foo appears in the exception');
 
 If the code in the C<...> is going to throw a stack trace with the arguments to
-each subroutine in its call stack, the test name, "foo appears in the
-exception" will itself be matched by the regex.  Instead, write this:
+each subroutine in its call stack (for example via C<Carp::confess>,
+the test name, "foo appears in the exception" will itself be matched by the
+regex.  Instead, write this:
 
-  my $exception = exception { ... };
-  like( $exception, qr/foo/, "foo appears in the exception" );
+  like( exception { ... }, qr/foo/, 'foo appears in the exception' );
+
+B<Achtung>: One final bad idea:
+
+  isnt( exception { ... }, undef, "my code died!");
+
+It's true that this tests that your code died, but you should really test that
+it died I<for the right reason>.  For example, if you make an unrelated mistake
+in the block, like using the wrong dereference, your test will pass even though
+the code to be tested isn't really run at all.  If you're expecting an
+inspectable exception with an identifier or class, test that.  If you're
+expecting a string exception, consider using C<like>.
 
 =head2 success
 
@@ -177,8 +216,8 @@ provide TAP output indicating if it did, or did not throw an exception.
 These provide an easy upgrade path for replacing existing unit tests based on
 C<Test::Exception>.
 
-RJBS does not using this except as a convenience while porting tests to use
-Test::Fatal's C<exception> routine.
+RJBS does not suggest using this except as a convenience while porting tests to
+use Test::Fatal's C<exception> routine.
 
   use Test::More tests => 2;
   use Test::Fatal qw(dies_ok lives_ok);
@@ -199,4 +238,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
